@@ -24,6 +24,7 @@
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/anjuta-pkg-config-chooser.h>
+#include <libanjuta/anjuta-pkg-config.h>
 #include <libanjuta/interfaces/ianjuta-iterable.h>
 #include <libanjuta/interfaces/ianjuta-document.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
@@ -35,13 +36,18 @@
 #include <libanjuta/interfaces/ianjuta-editor-assist.h>
 #include <libanjuta/interfaces/ianjuta-editor-glade-signal.h>
 #include <libanjuta/interfaces/ianjuta-editor-tip.h>
+#include <libanjuta/interfaces/ianjuta-editor-search.h>
 #include <libanjuta/interfaces/ianjuta-preferences.h>
 #include <libanjuta/interfaces/ianjuta-symbol.h>
+#include <libanjuta/interfaces/ianjuta-symbol-manager.h>
 #include <libanjuta/interfaces/ianjuta-language.h>
+#include <libanjuta/interfaces/ianjuta-indenter.h>
 
 #include "plugin.h"
 #include "cpp-java-utils.h"
 #include "cpp-java-indentation.h"
+#include "cpp-packages.h"
+
 
 /* Pixmaps */
 #define ANJUTA_PIXMAP_SWAP                "anjuta-swap"
@@ -55,15 +61,23 @@
 #define PREFS_BUILDER PACKAGE_DATA_DIR"/glade/anjuta-language-cpp-java.ui"
 #define ICON_FILE "anjuta-language-cpp-java-plugin.png"
 
-#define FIXME_DEFAULT_PACKAGE_VERSION		"1.0"
-
 /* Preferences keys */
 
 #define PREF_SCHEMA "org.gnome.anjuta.cpp"
 #define PREF_INDENT_AUTOMATIC "cpp-indent-automatic"
 #define PREF_INDENT_MODELINE "cpp-indent-modeline"
+#define PREF_USER_PACKAGES "cpp-user-packages"
 #define PREF_PROJECT_PACKAGES "cpp-load-project-packages"
 
+/* Callback generator defines */
+#define C_SEPARATOR "\n"
+#define C_BODY "\n{\n\n}\n"
+#define C_OFFSET 4
+
+#define CHDR_SEPARATOR " "
+#define CHDR_BODY ";\n"
+#define CHDR_OFFSET 1
+
 static gpointer parent_class;
 
 static void
@@ -425,82 +439,380 @@ language_support_get_signal_parameter (const gchar* type_name, GList** names)
 	return real_name;
 }
 
+static GString*
+language_support_generate_c_signature (const gchar* separator,
+									   const gchar* widget,
+									   GSignalQuery query,
+									   gboolean swapped,
+									   const gchar* handler)
+{
+	GList* names = NULL;
+	GString* str = g_string_new ("\n");
+	const gchar* widget_param = language_support_get_signal_parameter (widget,
+		                                                               &names);
+	int i;
+	g_string_append (str, g_type_name (query.return_type));
+	if (!swapped)
+		g_string_append_printf (str, "%s%s (%s *%s",
+									 separator, handler, widget, widget_param);
+	else
+		g_string_append_printf (str, "%s%s (gpointer user_data, %s *%s",
+									 separator, handler, widget, widget_param);
+
+	for (i = 0; i < query.n_params; i++)
+	{
+		const gchar* type_name = g_type_name (query.param_types[i]);
+		const gchar* param_name = language_support_get_signal_parameter (type_name,
+			                                                             &names);
+	
+		if (query.param_types[i] <= G_TYPE_DOUBLE)
+		{	                                                                
+			g_string_append_printf (str, ", %s %s", type_name, param_name);
+		}
+		else
+		{	                                                                
+			g_string_append_printf (str, ", %s *%s", type_name, param_name);
+		}
+	}
+	if (!swapped)
+		g_string_append (str, ", gpointer user_data)");
+	else
+		g_string_append (str, ")");
+
+	anjuta_util_glist_strings_free (names);
+
+	return str;
+}
+
+const gchar* SOURCE_EXT[] =
+{
+	".c",
+	".cc",
+	".C",
+	".cpp",
+	".cxx",
+	".ccg",
+	NULL
+};
+
+const gchar* HEADER_EXT[] =
+{
+	".h",
+	".hh",
+	".H",
+	".hpp",
+	".hxx",
+	".hg",
+	NULL
+};
+
+static GFile*
+language_support_get_header_file (IAnjutaEditor* editor)
+{
+	GFile *file = ianjuta_file_get_file (IANJUTA_FILE (editor), NULL);
+	GFile *parent = g_file_get_parent (file);
+	gchar *parent_uri = g_file_get_uri (parent);
+	gchar *basename = g_file_get_basename (file);
+	g_object_unref (file);
+	g_object_unref (parent);
+	gchar *ext = strstr (basename, ".");
+	GFile *ret = NULL;
+
+	if (ext)
+	{
+		int i;
+		for (i = 0; SOURCE_EXT[i] != NULL; i++)
+		{
+			if (g_str_equal (ext, SOURCE_EXT[i]))
+			{
+				int j;
+				for (j = 0; HEADER_EXT[j] != NULL; j++)
+				{
+					gchar* filename;
+					gchar* uri;
+					GFile* new_file;
+					*ext = '\0';
+					filename = g_strdup_printf ("%s%s", basename, HEADER_EXT[j]);
+					uri = g_build_filename (parent_uri, filename, NULL);
+					new_file = g_file_new_for_uri (uri);
+					g_free (uri);
+					g_free(filename);
+					if (g_file_query_exists (new_file, NULL))
+					{
+            			ret = new_file;
+						goto end;
+					}
+					g_object_unref (new_file);
+				}
+				break;
+			}
+			if (g_str_equal (ext, HEADER_EXT[i]))
+			{
+				int j;
+				for (j = 0; SOURCE_EXT[j] != NULL; j++)
+				{
+					gchar* filename;
+					gchar* uri;
+					GFile* new_file;
+					*ext = '\0';
+					filename = g_strdup_printf ("%s%s", basename, SOURCE_EXT[j]);
+					uri = g_build_filename (parent_uri, filename, NULL);
+					new_file = g_file_new_for_uri (uri);
+					g_free (uri);
+					g_free(filename);
+					if (g_file_query_exists (new_file, NULL))
+					{
+            			ret = new_file;
+						goto end;
+					}
+					g_object_unref (new_file);
+				}
+				break;
+			}
+		}
+	}
+
+end:
+	g_free(basename);
+	g_free (parent_uri);
+
+  return ret;
+}
+
+static IAnjutaEditor*
+language_support_get_editor_from_file (CppJavaPlugin* lang_plugin,
+													  GFile *file)
+{
+	IAnjutaDocumentManager *document_manager = anjuta_shell_get_interface (
+										  ANJUTA_PLUGIN (lang_plugin)->shell,
+										  IAnjutaDocumentManager,
+										  NULL);
+
+	IAnjutaDocument *document = ianjuta_document_manager_find_document_with_file
+                                                        (document_manager,
+                                                         file,
+                                                         NULL);
+
+	return IANJUTA_EDITOR (document);
+}
+
+static IAnjutaIterable*
+language_support_get_mark_position (IAnjutaEditor* editor, gchar* mark)
+{
+	IAnjutaEditorCell *search_start = IANJUTA_EDITOR_CELL (
+										ianjuta_editor_get_start_position(editor, NULL));
+	IAnjutaEditorCell *search_end = IANJUTA_EDITOR_CELL (
+										ianjuta_editor_get_end_position(editor, NULL));
+	IAnjutaEditorCell *result_start;
+	IAnjutaEditorCell *result_end;
+
+	ianjuta_editor_search_forward (IANJUTA_EDITOR_SEARCH (editor),
+								   mark, FALSE,
+								   search_start, search_end,
+								   &result_start,
+								   &result_end, NULL);
+
+	g_object_unref (result_start);
+
+	return IANJUTA_ITERABLE (result_end);
+}
+
+static IAnjutaIterable*
+language_support_find_symbol (CppJavaPlugin* lang_plugin,
+							  IAnjutaEditor* editor,
+							  const gchar* handler)
+{
+	IAnjutaSymbolManager *isymbol_manager = anjuta_shell_get_interface (
+										  ANJUTA_PLUGIN (lang_plugin)->shell,
+										  IAnjutaSymbolManager,
+										  NULL);
+
+	IAnjutaSymbolQuery *symbol_query = ianjuta_symbol_manager_create_query (
+										 isymbol_manager,
+                                     	 IANJUTA_SYMBOL_QUERY_SEARCH_FILE,
+                                     	 IANJUTA_SYMBOL_QUERY_DB_PROJECT,
+                                     	 NULL);
+	IAnjutaSymbolField field = IANJUTA_SYMBOL_FIELD_FILE_POS;
+	ianjuta_symbol_query_set_fields (symbol_query, 1, &field, NULL);
+
+	GFile* file = ianjuta_file_get_file (IANJUTA_FILE (editor),
+                                 		 NULL);
+
+	IAnjutaIterable *iter = ianjuta_symbol_query_search_file (symbol_query,
+                         				            		  handler, file, NULL);
+
+	g_object_unref (file);
+	g_object_unref (symbol_query);
+
+	return iter;
+}
+
+static gboolean
+language_support_get_callback_strings (gchar** separator,
+									   gchar** body,
+									   gint* offset,
+									   CppFileType filetype)
+{
+	switch (filetype)
+	{
+		case LS_FILE_C:
+		{
+			*separator = C_SEPARATOR;
+			*body = C_BODY;
+			*offset = C_OFFSET;
+			break;
+		}
+		case LS_FILE_CHDR:
+		{
+			*separator = CHDR_SEPARATOR;
+			*body = CHDR_BODY;
+			*offset = CHDR_OFFSET;
+			break;
+		}
+		default:
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+static IAnjutaIterable*
+language_support_get_header_editor_and_mark (CppJavaPlugin* lang_plugin,
+											 IAnjutaEditor* editor,
+											 gchar* mark,
+											 IAnjutaEditor** header_editor)
+{
+	IAnjutaIterable* mark_position;
+	GFile *header_file = language_support_get_header_file (editor);
+
+	/* Yes, we have a header */
+	if (header_file)
+	{
+		*header_editor = language_support_get_editor_from_file (lang_plugin,
+																header_file);
+		/* We'll now search for the mark */
+
+		mark_position = language_support_get_mark_position (*header_editor, mark);
+
+		g_object_unref (header_file);
+	}
+
+	return mark_position;
+}
+
+static void
+language_support_add_c_callback (CppJavaPlugin* lang_plugin,
+               					 IAnjutaEditor* editor,
+               					 IAnjutaIterable* position,
+				                 GStrv split_signal_data,
+								 CppFileType filetype)
+{
+	GSignalQuery query;
+
+	gchar* separator;
+	gchar* body;
+	gint offset;
+
+	const gchar* widget = split_signal_data[0];
+	const gchar* signal = split_signal_data[1];
+	const gchar* handler = split_signal_data[2];
+	//const gchar* user_data = split_signal_data[3]; // Currently unused
+	gboolean swapped = g_str_equal (split_signal_data[4], "1");
+
+	GType type = g_type_from_name (widget);
+	guint id = g_signal_lookup (signal, type);
+
+	g_signal_query (id, &query);
+
+
+	if (!language_support_get_callback_strings (&separator, &body, &offset, filetype))
+		return;
+
+
+
+	GString* str = language_support_generate_c_signature (separator, widget,
+														  query, swapped, handler);
+
+	g_string_append (str, body);
+
+	ianjuta_editor_insert (editor, position,
+		                   str->str, -1, NULL);
+
+	/* Code was inserted, we'll now check if we should add a prototype to the header */
+	if (filetype == LS_FILE_C)
+	{
+		IAnjutaEditor* header_editor;
+		IAnjutaIterable* mark_position;
+		mark_position = language_support_get_header_editor_and_mark (lang_plugin,
+																	 editor,
+																	 "/* Callbacks */", 
+																	 &header_editor);
+		if (mark_position)
+		{
+			/* Check if there's a the prototype to the header */
+			IAnjutaIterable* symbol;
+			symbol = language_support_find_symbol (lang_plugin, header_editor, handler);
+
+			if (symbol)
+			{
+				g_object_unref (symbol);
+			} else {
+				/* Add prototype to the header */
+				language_support_add_c_callback (lang_plugin, header_editor, mark_position,
+												 split_signal_data, LS_FILE_CHDR);
+
+			}
+
+			g_object_unref (mark_position);
+		}
+	}
+
+	gchar *string = g_string_free (str, FALSE);
+	/* Emit code-added signal, so symbols will be updated */
+	g_signal_emit_by_name (G_OBJECT (editor), "code-added", position, string);
+
+	if (string) g_free (string);
+
+	/* Will now set the caret position offset */
+	ianjuta_editor_goto_line (editor,
+							  ianjuta_editor_get_line_from_position (
+											editor, position, NULL) + offset, NULL);
+}
+
 static void
 on_glade_drop (IAnjutaEditor* editor,
                IAnjutaIterable* iterator,
                const gchar* signal_data,
                CppJavaPlugin* lang_plugin)
 {
-	GSignalQuery query;
-	GType type;
-	guint id;
-	
-	const gchar* widget;
-	const gchar* signal;
-	const gchar* handler;
-	const gchar* user_data;
-	gboolean swapped;
+	GStrv split_signal_data = g_strsplit(signal_data, ":", 5);
+	char *handler = split_signal_data[2];
+	/**
+	 * Split signal data format:
+	 * widget = split_signaldata[0];
+	 * signal = split_signaldata[1];
+	 * handler = split_signaldata[2];
+	 * user_data = split_signaldata[3];
+	 * swapped = g_str_equal (split_signaldata[4], "1");
+	 */
 
-	GStrv data = g_strsplit(signal_data, ":", 5);
-	
-	widget = data[0];
-	signal = data[1];
-	handler = data[2];
-	user_data = data[3];
-	swapped = g_str_equal (data[4], "1");
-	
-	type = g_type_from_name (widget);
-	id = g_signal_lookup (signal, type);
-
-	g_signal_query (id, &query);
-	
-	switch (lang_plugin->filetype)
+	IAnjutaIterable *iter;
+	iter = language_support_find_symbol (lang_plugin,
+										 IANJUTA_EDITOR (lang_plugin->current_editor),
+										 handler);
+	if (iter == NULL)
 	{
-		case LS_FILE_C:
-		{
-			GList* names = NULL;
-			GString* str = g_string_new ("\nstatic ");
-			const gchar* widget_param = language_support_get_signal_parameter (widget,
-			                                                                   &names);
-			int i;
-			g_string_append (str, g_type_name (query.return_type));
-			if (!swapped)
-				g_string_append_printf (str, "\n%s (%s *%s", handler, widget, widget_param);
-			else
-				g_string_append_printf (str, "\n%s (gpointer user_data, %s *%s", handler, widget, widget_param);				
-			for (i = 0; i < query.n_params; i++)
-			{
-				const gchar* type_name = g_type_name (query.param_types[i]);
-				const gchar* param_name = language_support_get_signal_parameter (type_name,
-				                                                                 &names);
-				
-				if (query.param_types[i] <= G_TYPE_DOUBLE)
-				{	                                                                
-					g_string_append_printf (str, ", %s %s", type_name, param_name);
-				}
-				else
-				{	                                                                
-					g_string_append_printf (str, ", %s *%s", type_name, param_name);
-				}
-			}
-			if (!swapped)
-				g_string_append (str, ", gpointer user_data)");
-			else
-				g_string_append (str, ")");
-
-			g_string_append (str, "\n{\n\n}\n");
-
-			ianjuta_editor_insert (editor, iterator,
-			                       str->str, -1, NULL);
-			g_string_free (str, TRUE);
-			anjuta_util_glist_strings_free (names);
-			break;
-		}
-		case LS_FILE_CHDR:
-		default:
-			break;
+		language_support_add_c_callback (lang_plugin, editor, iterator, split_signal_data,
+										 lang_plugin->filetype);
+	} else {
+		/* Symbol found, going there */
+		ianjuta_editor_goto_line (editor, ianjuta_symbol_get_int (
+															IANJUTA_SYMBOL (iter),
+															IANJUTA_SYMBOL_FIELD_FILE_POS,
+															NULL), NULL);		
+		g_object_unref (iter);
 	}
-	g_strfreev (data);
+	g_strfreev (split_signal_data);
 }
 
 /* Enable/Disable language-support */
@@ -552,7 +864,8 @@ install_support (CppJavaPlugin *lang_plugin)
 	init_file_type (lang_plugin);
 
 	
-	if (!g_str_equal (lang_plugin->current_language, "Vala"))
+	if (g_str_equal (lang_plugin->current_language, "C" ) ||
+	    g_str_equal (lang_plugin->current_language, "C++"))
 	{
 		CppJavaAssist *assist;
 
@@ -575,8 +888,11 @@ install_support (CppJavaPlugin *lang_plugin)
 			                  "drop", G_CALLBACK (on_glade_drop),
 			                  lang_plugin);
 		}
-	}	
-		
+
+		lang_plugin->packages = cpp_packages_new (ANJUTA_PLUGIN (lang_plugin));
+		cpp_packages_load(lang_plugin->packages, FALSE);
+	}
+	
 	lang_plugin->support_installed = TRUE;
 }
 
@@ -613,8 +929,11 @@ uninstall_support (CppJavaPlugin *lang_plugin)
 	                                      on_glade_drop_possible, lang_plugin);
 	g_signal_handlers_disconnect_by_func (lang_plugin->current_editor,
 	                                      on_glade_drop, lang_plugin);
-	
-	
+	if (lang_plugin->packages)
+	{
+		g_object_unref (lang_plugin->packages);
+		lang_plugin->packages = NULL;
+	}
 	lang_plugin->support_installed = FALSE;
 }
 
@@ -663,36 +982,10 @@ on_value_removed_current_editor (AnjutaPlugin *plugin, const gchar *name,
 	lang_plugin->current_editor = NULL;
 }
 
-const gchar* SOURCE_EXT[] =
-{
-	".c",
-	".cc",
-	".C",
-	".cpp",
-	".cxx",
-	".ccg",
-	NULL
-};
-
-const gchar* HEADER_EXT[] =
-{
-	".h",
-	".hh",
-	".H",
-	".hpp",
-	".hxx",
-	".hg",
-	NULL
-};
-
 static void
 on_swap_activate (GtkAction* action, gpointer data)
 {
 	GFile* file;
-	GFile* parent;
-	gchar* parent_uri;
-	gchar* basename;
-	gchar* ext;
 	CppJavaPlugin *lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (data);
 	IAnjutaDocumentManager* docman =
 		anjuta_shell_get_interface (ANJUTA_PLUGIN(lang_plugin)->shell,
@@ -703,75 +996,18 @@ on_swap_activate (GtkAction* action, gpointer data)
 	
 	file = ianjuta_file_get_file (IANJUTA_FILE (lang_plugin->current_editor),
 								  NULL);
-	parent = g_file_get_parent (file);
-	parent_uri = g_file_get_uri (parent);
-	basename = g_file_get_basename (file);
-	g_object_unref (file);
-	g_object_unref (parent);
-	ext = strstr (basename, ".");
-	if (ext)
+
+	GFile* new_file = language_support_get_header_file (
+										IANJUTA_EDITOR (lang_plugin->current_editor));
+
+	if (g_file_query_exists (new_file, NULL))
 	{
-		int i;
-		for (i = 0; SOURCE_EXT[i] != NULL; i++)
-		{
-			if (g_str_equal (ext, SOURCE_EXT[i]))
-			{
-				int j;
-				for (j = 0; HEADER_EXT[j] != NULL; j++)
-				{
-					gchar* filename;
-					gchar* uri;
-					GFile* new_file;
-					*ext = '\0';
-					filename = g_strdup_printf ("%s%s", basename, HEADER_EXT[j]);
-					uri = g_build_filename (parent_uri, filename, NULL);
-					new_file = g_file_new_for_uri (uri);
-					g_free (uri);
-					g_free(filename);
-					if (g_file_query_exists (new_file, NULL))
-					{
-						ianjuta_document_manager_goto_file_line (docman,
-																 new_file,
-																 -1,
-																 NULL);
-						g_object_unref (new_file);
-						break;
-					}
-					g_object_unref (new_file);
-				}
-				break;
-			}
-			if (g_str_equal (ext, HEADER_EXT[i]))
-			{
-				int j;
-				for (j = 0; SOURCE_EXT[j] != NULL; j++)
-				{
-					gchar* filename;
-					gchar* uri;
-					GFile* new_file;
-					*ext = '\0';
-					filename = g_strdup_printf ("%s%s", basename, SOURCE_EXT[j]);
-					uri = g_build_filename (parent_uri, filename, NULL);
-					new_file = g_file_new_for_uri (uri);
-					g_free (uri);
-					g_free(filename);
-					if (g_file_query_exists (new_file, NULL))
-					{
-						ianjuta_document_manager_goto_file_line (docman,
-																 new_file,
-																 -1,
-																 NULL);
-						g_object_unref (new_file);
-						break;
-					}
-					g_object_unref (new_file);
-				}
-				break;
-			}
-		}
+		ianjuta_document_manager_goto_file_line (docman,
+												 new_file,
+												 -1,
+												 NULL);
+		g_object_unref (new_file);
 	}
-	g_free(basename);
-	g_free (parent_uri);
 }
 
 static void
@@ -782,7 +1018,7 @@ on_auto_indent (GtkAction *action, gpointer data)
 	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (data);
 	editor = IANJUTA_EDITOR (lang_plugin->current_editor);
 
-	cpp_auto_indentation (editor, lang_plugin);
+	cpp_auto_indentation (editor, lang_plugin, NULL, NULL);
 }
 
 /* Automatic comments */
@@ -1031,6 +1267,7 @@ cpp_java_plugin_activate_plugin (AnjutaPlugin *plugin)
 								 on_value_added_current_editor,
 								 on_value_removed_current_editor,
 								 plugin);
+
 	initialized = FALSE;
 	return TRUE;
 }
@@ -1087,6 +1324,7 @@ cpp_java_plugin_instance_init (GObject *obj)
 	plugin->uiid = 0;
 	plugin->assist = NULL;
 	plugin->settings = g_settings_new (PREF_SCHEMA);
+	plugin->packages = NULL;
 }
 
 static void
@@ -1110,28 +1348,85 @@ cpp_java_plugin_class_init (GObjectClass *klass)
 
 static void
 on_autocompletion_toggled (GtkToggleButton* button,
-                           GtkBuilder* bxml)
+                           CppJavaPlugin* plugin)
 {
 	GtkWidget* widget;
 	gboolean sensitive = gtk_toggle_button_get_active (button);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (bxml, PREF_WIDGET_SPACE));
+	widget = GTK_WIDGET (gtk_builder_get_object (plugin->bxml, PREF_WIDGET_SPACE));
 	gtk_widget_set_sensitive (widget, sensitive);
-	widget = GTK_WIDGET (gtk_builder_get_object (bxml, PREF_WIDGET_BRACE));
+	widget = GTK_WIDGET (gtk_builder_get_object (plugin->bxml, PREF_WIDGET_BRACE));
 	gtk_widget_set_sensitive (widget, sensitive);
 }
 
+static void 
+cpp_java_plugin_select_user_packages (CppJavaPlugin* plugin,
+                                      AnjutaPkgConfigChooser* chooser)
+{
+	gchar* user_packages = g_settings_get_string (plugin->settings,
+	                                              PREF_USER_PACKAGES);
+	GStrv pkgs = g_strsplit (user_packages, ";", -1);
+	gchar** pkg;
+	GList* packages = NULL;
+	for (pkg = pkgs; *pkg != NULL; pkg++)
+	{
+		packages = g_list_append (packages, *pkg);
+	}
+	anjuta_pkg_config_chooser_set_active_packages (chooser,
+	                                               packages);
+	g_strfreev (pkgs);
+	g_free (user_packages);
+	g_list_free (packages);
+}
+
 static void
 on_project_packages_toggled (GtkToggleButton* button,
-                             GtkBuilder* bxml)
+                             CppJavaPlugin* plugin)
 {
 	GtkWidget* pkg_config;
-	gboolean sensitive = !gtk_toggle_button_get_active (button);
-	pkg_config = GTK_WIDGET (gtk_builder_get_object (bxml, PREF_WIDGET_PKG_CONFIG));
+	gboolean active = gtk_toggle_button_get_active (button);
+	pkg_config = GTK_WIDGET (gtk_builder_get_object (plugin->bxml, PREF_WIDGET_PKG_CONFIG));
 
-	gtk_widget_set_sensitive (pkg_config, sensitive);
-	anjuta_pkg_config_chooser_show_active_only (ANJUTA_PKG_CONFIG_CHOOSER (pkg_config),
-	                                            !sensitive);
+	gtk_widget_set_sensitive (pkg_config, !active);
+
+	anjuta_pkg_config_chooser_set_active_packages (ANJUTA_PKG_CONFIG_CHOOSER (pkg_config),
+	                                               NULL);
+	if (!active)
+	{
+		anjuta_pkg_config_chooser_show_active_only (ANJUTA_PKG_CONFIG_CHOOSER (pkg_config),
+		                                            FALSE);
+		cpp_java_plugin_select_user_packages (plugin, ANJUTA_PKG_CONFIG_CHOOSER (pkg_config));
+		cpp_packages_load (plugin->packages, TRUE);
+	}
+	else
+	{
+		anjuta_pkg_config_chooser_set_active_packages (ANJUTA_PKG_CONFIG_CHOOSER (pkg_config),
+		                                               NULL);
+		anjuta_pkg_config_chooser_show_active_only (ANJUTA_PKG_CONFIG_CHOOSER (pkg_config),
+		                                            TRUE);
+
+	}
+}
+
+static void
+cpp_java_plugin_update_user_packages (CppJavaPlugin* plugin,
+                                      AnjutaPkgConfigChooser* chooser)
+{
+	GList* pkg;
+	GList* packages = anjuta_pkg_config_chooser_get_active_packages (chooser);
+	GString* pkg_string = g_string_new (NULL);
+
+	for (pkg = packages; pkg != NULL; pkg = g_list_next (pkg))
+	{
+		if (strlen (pkg_string->str))
+		{
+				pkg_string = g_string_append_c (pkg_string, ';');
+		}
+		pkg_string = g_string_append (pkg_string, pkg->data);
+	}
+	g_settings_set_string (plugin->settings, PREF_USER_PACKAGES,
+	                       pkg_string->str);
+	g_string_free (pkg_string, TRUE);
 }
 
 static void
@@ -1139,20 +1434,13 @@ on_package_activated (AnjutaPkgConfigChooser *self, const gchar* package,
     				  gpointer data)
 {
 	CppJavaPlugin* plugin;
-	IAnjutaSymbolManager *isymbol_manager;
 
 	plugin = ANJUTA_PLUGIN_CPP_JAVA (data);
 
-	DEBUG_PRINT ("activated %s", package);
-	isymbol_manager = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell,
-												IAnjutaSymbolManager,
-												NULL);
-	
-	ianjuta_symbol_manager_activate_package (isymbol_manager, 
-	    									 package, 
-    							  			 FIXME_DEFAULT_PACKAGE_VERSION,
-    							  			 NULL);
+	g_message ("Activate package");
 	
+	cpp_java_plugin_update_user_packages (plugin, self);
+	cpp_packages_load (plugin->packages, TRUE);
 }
 
 static void
@@ -1161,7 +1449,8 @@ on_package_deactivated (AnjutaPkgConfigChooser *self, const gchar* package,
 {
 	CppJavaPlugin* plugin;
 	IAnjutaSymbolManager *isymbol_manager;
-
+	gchar* version;
+	
 	plugin = ANJUTA_PLUGIN_CPP_JAVA (data);
 
 	DEBUG_PRINT ("deactivated %s", package);
@@ -1169,12 +1458,18 @@ on_package_deactivated (AnjutaPkgConfigChooser *self, const gchar* package,
 	isymbol_manager = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell,
 												IAnjutaSymbolManager,
 												NULL);
-	ianjuta_symbol_manager_deactivate_package (isymbol_manager, 
-	    									 package, 
-    							  			 FIXME_DEFAULT_PACKAGE_VERSION,
-    							  			 NULL);
-}
+	version = anjuta_pkg_config_get_version (package);
+	if (version)
+	{
+		ianjuta_symbol_manager_deactivate_package (isymbol_manager, 
+		                                           package, 
+		                                           version,
+		                                           NULL);
+	}
+	g_free (version);
 
+	cpp_java_plugin_update_user_packages (plugin, self);
+}
 static void
 ipreferences_merge (IAnjutaPreferences* ipref, AnjutaPreferences* prefs,
 					GError** e)
@@ -1197,25 +1492,29 @@ ipreferences_merge (IAnjutaPreferences* ipref, AnjutaPreferences* prefs,
 	                                     ICON_FILE);
 	toggle = GTK_WIDGET (gtk_builder_get_object (plugin->bxml, PREF_WIDGET_AUTO));
 	g_signal_connect (toggle, "toggled", G_CALLBACK (on_autocompletion_toggled),
-	                  plugin->bxml);
-	on_autocompletion_toggled (GTK_TOGGLE_BUTTON (toggle), plugin->bxml);
+	                  plugin);
+	on_autocompletion_toggled (GTK_TOGGLE_BUTTON (toggle), plugin);
 
 	toggle = GTK_WIDGET (gtk_builder_get_object (plugin->bxml, PREF_WIDGET_PACKAGES));
 	g_signal_connect (toggle, "toggled", G_CALLBACK (on_project_packages_toggled),
-	                  plugin->bxml);
-	on_autocompletion_toggled (GTK_TOGGLE_BUTTON (toggle), plugin->bxml);
-	on_project_packages_toggled (GTK_TOGGLE_BUTTON (toggle), plugin->bxml);
+	                  plugin);
+	on_project_packages_toggled (GTK_TOGGLE_BUTTON (toggle), plugin);
 	
 	pkg_config = GTK_WIDGET (gtk_builder_get_object (plugin->bxml, PREF_WIDGET_PKG_CONFIG));
 	anjuta_pkg_config_chooser_show_active_column (ANJUTA_PKG_CONFIG_CHOOSER (pkg_config), 
 	    										  TRUE);
+	
 	g_signal_connect (G_OBJECT (pkg_config), "package-activated",
-					  G_CALLBACK (on_package_activated), plugin);
+	                  G_CALLBACK (on_package_activated), plugin);
 
 	g_signal_connect (G_OBJECT (pkg_config), "package-deactivated",
 					  G_CALLBACK (on_package_deactivated), plugin);
 	
-	gtk_widget_show_all (pkg_config);
+	if (!g_settings_get_boolean (plugin->settings,
+	                             PREF_PROJECT_PACKAGES))
+		cpp_java_plugin_select_user_packages (plugin, ANJUTA_PKG_CONFIG_CHOOSER (pkg_config));
+	
+	gtk_widget_show (pkg_config);
 }
 
 static void
@@ -1234,8 +1533,28 @@ ipreferences_iface_init (IAnjutaPreferencesIface* iface)
 	iface->unmerge = ipreferences_unmerge;	
 }
 
+static void
+iindenter_indent (IAnjutaIndenter* indenter,
+                  IAnjutaIterable* start,
+                  IAnjutaIterable* end,
+                  GError** e)
+{
+	CppJavaPlugin* plugin = ANJUTA_PLUGIN_CPP_JAVA (indenter);
+
+	cpp_auto_indentation (IANJUTA_EDITOR (plugin->current_editor),
+	                      plugin,
+	                      start, end);
+}
+
+static void
+iindenter_iface_init (IAnjutaIndenterIface* iface)
+{
+	iface->indent = iindenter_indent;
+}
+
 ANJUTA_PLUGIN_BEGIN (CppJavaPlugin, cpp_java_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE(ipreferences, IANJUTA_TYPE_PREFERENCES);
+ANJUTA_PLUGIN_ADD_INTERFACE(iindenter, IANJUTA_TYPE_INDENTER);
 ANJUTA_PLUGIN_END;
 
 ANJUTA_SIMPLE_PLUGIN (CppJavaPlugin, cpp_java_plugin);
