@@ -207,14 +207,14 @@ module_filter_func (GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
 	GbfTreeData *data = NULL;
 	gboolean visible = FALSE;
 	AnjutaProjectNode *node;
-
+	
 	gtk_tree_model_get (model, iter,
 						GBF_PROJECT_MODEL_COLUMN_DATA, &data, -1);
 	node = data == NULL ? NULL : gbf_tree_data_get_node (data);
 	if (node != NULL)
 	{
 		AnjutaProjectNodeType type = anjuta_project_node_get_node_type (node);
-		
+
 		visible = (type == ANJUTA_PROJECT_MODULE) || (type == ANJUTA_PROJECT_PACKAGE);
 	}
 	
@@ -222,57 +222,19 @@ module_filter_func (GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
 }
 
 static void 
-setup_nodes_treeview (GbfProjectModel         *model,
-						GtkWidget              *view,
+setup_nodes_treeview (GbfProjectView           *view,
+						GbfProjectView		   *parent,
+                  	    GtkTreePath            *root,
 						GtkTreeModelFilterVisibleFunc func,
 						gpointer               data,
 						GtkTreeIter            *selected)
 {
-	GtkTreeModel *filter;
-	GtkTreeIter iter_filter;
-	GtkTreePath *path;
-
-	g_return_if_fail (model != NULL);
 	g_return_if_fail (view != NULL && GBF_IS_PROJECT_VIEW (view));
+	g_return_if_fail (parent != NULL);
 
-	filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
-	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
-											func, data, NULL);
-	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (filter));
-	g_object_unref (filter);
-
-	/* select default node */
-	if (selected && gtk_tree_model_filter_convert_child_iter_to_iter (
-			GTK_TREE_MODEL_FILTER (filter), &iter_filter, selected))
-	{
-		path = gtk_tree_model_get_path (filter, &iter_filter);
-		gtk_tree_view_expand_to_path (GTK_TREE_VIEW (view), path);
-	}
-	else
-	{
-		GtkTreePath *root_path;
-
-		gtk_tree_view_expand_all (GTK_TREE_VIEW (view));
-		root_path = gbf_project_model_get_project_root (model);
-		if (root_path)
-		{
-			path = gtk_tree_model_filter_convert_child_path_to_path (
-						GTK_TREE_MODEL_FILTER (filter), root_path);
-			gtk_tree_path_free (root_path);
-		}
-		else
-		{
-			path = gtk_tree_path_new_first ();
-		}
-	}
-
-	if (path)
-	{
-		gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
-		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), path, NULL,
-									TRUE, 0.5, 0.0);
-		gtk_tree_path_free (path);
-	}
+	gbf_project_view_set_parent_view (view, parent, root);
+	gbf_project_view_set_visible_func (view, func, data, NULL);
+	gbf_project_view_set_cursor_to_iter (view, selected);
 }
 
 static void
@@ -390,6 +352,7 @@ add_entry (IAnjutaProject *project, AnjutaProjectNode *node, AnjutaProjectProper
 	GtkTreeIter iter;
 	GList *item;
 	gchar *tooltip = NULL;
+	gboolean editable = TRUE;
 
 	if (prop->native != NULL)
 	{
@@ -400,9 +363,11 @@ add_entry (IAnjutaProject *project, AnjutaProjectNode *node, AnjutaProjectProper
 		label = gtk_label_new (_(prop->name));
 	}
 
+	editable = prop->flags & ANJUTA_PROJECT_PROPERTY_READ_ONLY ? FALSE : TRUE;
+	
 	if (prop->detail != NULL)
 	{
-		if (prop->flags & ANJUTA_PROJECT_PROPERTY_READ_ONLY)
+		if (!editable)
 		{
 			tooltip = g_strconcat (_(prop->detail), _(" This property is not modifiable."), NULL);
 		}
@@ -425,12 +390,21 @@ add_entry (IAnjutaProject *project, AnjutaProjectNode *node, AnjutaProjectProper
 	{
 	case ANJUTA_PROJECT_PROPERTY_STRING:
 	case ANJUTA_PROJECT_PROPERTY_LIST:
-		entry = gtk_entry_new ();
-		gtk_entry_set_text (GTK_ENTRY (entry), prop->value != NULL ? prop->value : "");
+		if (editable)
+		{
+			entry = gtk_entry_new ();
+			gtk_entry_set_text (GTK_ENTRY (entry), prop->value != NULL ? prop->value : "");
+		}
+		else
+		{
+			entry = gtk_label_new (prop->value != NULL ? prop->value : "");
+			gtk_misc_set_alignment (GTK_MISC (entry), 0, 0.5);
+		}
 		break;
 	case ANJUTA_PROJECT_PROPERTY_BOOLEAN:
 		entry = gtk_check_button_new ();
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry), (prop->value != NULL) && (*prop->value == '1'));
+		gtk_widget_set_sensitive (entry, editable);
 		break;
 	case ANJUTA_PROJECT_PROPERTY_MAP:
 			model = GTK_TREE_MODEL (gtk_list_store_newv (LIST_COLUMNS_NB, column_type));
@@ -476,6 +450,7 @@ add_entry (IAnjutaProject *project, AnjutaProjectNode *node, AnjutaProjectProper
 			gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 			gtk_tree_view_append_column (GTK_TREE_VIEW (view), column);
 
+			gtk_widget_set_sensitive (entry, editable);
 			gtk_container_add (GTK_CONTAINER (entry), view);
 			
 			break;
@@ -691,21 +666,24 @@ on_properties_dialog_response (GtkWidget *dialog,
 			{
 			case ANJUTA_PROJECT_PROPERTY_STRING:
 			case ANJUTA_PROJECT_PROPERTY_LIST:
-				text = gtk_entry_get_text (GTK_ENTRY (entry->entry));
-				if (*text == '\0')
+				if (GTK_IS_ENTRY (entry->entry))
 				{
-					if ((prop->value != NULL) && (*prop->value != '\0'))
+					text = gtk_entry_get_text (GTK_ENTRY (entry->entry));
+					if (*text == '\0')
 					{
-						/* Remove */
-						ianjuta_project_set_property (table->project->project, table->node, prop, NULL, NULL);
+						if ((prop->value != NULL) && (*prop->value != '\0'))
+						{
+							/* Remove */
+							ianjuta_project_set_property (table->project->project, table->node, prop, NULL, NULL);
+						}
 					}
-				}
-				else
-				{
-					if (g_strcmp0 (prop->value, text) != 0)
+					else
 					{
-						/* Modified */
-						ianjuta_project_set_property (table->project->project, table->node, prop, text, NULL);
+						if (g_strcmp0 (prop->value, text) != 0)
+						{
+							/* Modified */
+							ianjuta_project_set_property (table->project->project, table->node, prop, text, NULL);
+						}
 					}
 				}
 				break;
@@ -798,8 +776,9 @@ pm_project_create_properties_dialog (AnjutaPmProject *project, GtkWindow *parent
 	g_object_unref (bxml);
 
 	/* Add tree view */
-	setup_nodes_treeview (gbf_project_view_get_model (view),
-							node_view,
+	setup_nodes_treeview (GBF_PROJECT_VIEW (node_view),
+	                        view,
+	                        NULL,
 							is_project_node_but_shortcut,
 							NULL,
 							selected);
@@ -900,6 +879,7 @@ anjuta_pm_project_new_group (ProjectManagerPlugin *plugin, GtkWindow *parent, Gt
 	GtkBuilder *gui;
 	GtkWidget *dialog, *group_name_entry, *ok_button;
 	GtkWidget *groups_view;
+	GtkTreePath *root;
 	gint response;
 	gboolean finished = FALSE;
 	AnjutaProjectNode *new_group = NULL;
@@ -926,11 +906,14 @@ anjuta_pm_project_new_group (ProjectManagerPlugin *plugin, GtkWindow *parent, Gt
 	else
 		gtk_widget_set_sensitive (ok_button, FALSE);
 
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							groups_view,
+	root = gbf_project_model_get_project_root (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (groups_view),
+	                        plugin->view,
+	                      	root,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_GROUP),
 							selected);
+	gtk_tree_path_free (root);
 	gtk_widget_show (groups_view);
 
 	if (parent)
@@ -1041,6 +1024,7 @@ anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 	GtkTreeViewColumn* column_filename;
 	GList* new_sources = NULL;
 	GList* uri_node;
+	GtkTreePath *root;
 
 	g_return_val_if_fail (plugin->project != NULL, NULL);
 
@@ -1098,11 +1082,14 @@ anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 	/*project_root = g_file_get_uri (anjuta_project_node_get_file (anjuta_pm_project_get_root (project)));
 	g_object_set_data_full (G_OBJECT (browse_button), "root", project_root, g_free);*/
 
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							targets_view,
+	root = gbf_project_model_get_project_root_group (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (targets_view),
+	                        plugin->view,
+	                       	root,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_SOURCE),
 							default_parent);
+	gtk_tree_path_free (root);
 	gtk_widget_show (targets_view);
 
 	if (top_window) {
@@ -1193,7 +1180,7 @@ anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 			else 
 			{
 				error_dialog (top_window, _("Cannot add source files"),
-						"%s", _("The selected node cannot contains source files."));
+						"%s", _("The selected node cannot contain source files."));
 			}
 			break;
 		}
@@ -1223,7 +1210,7 @@ enum {
 
 /* create a tree model with the target types */
 static GtkListStore *
-build_types_store (AnjutaPmProject *project)
+build_types_store (AnjutaPmProject *project, AnjutaProjectNodeType store_type)
 {
     GtkListStore *store;
     GtkTreeIter iter;
@@ -1241,8 +1228,8 @@ build_types_store (AnjutaPmProject *project)
         const gchar *name;
         AnjutaProjectNodeType type;
 
-        type = anjuta_project_node_info_type ((AnjutaProjectNodeInfo *)node->data);
-        if (type & ANJUTA_PROJECT_TARGET)
+		type = anjuta_project_node_info_type ((AnjutaProjectNodeInfo *)node->data);
+        if (((store_type == 0) || ((type & ANJUTA_PROJECT_TYPE_MASK) == store_type)) && !(type & ANJUTA_PROJECT_READ_ONLY))
         {
             name = anjuta_project_node_info_name ((AnjutaProjectNodeInfo *)node->data);
             pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default(),
@@ -1275,6 +1262,7 @@ anjuta_pm_project_new_target (ProjectManagerPlugin *plugin,
 	GtkBuilder *gui;
 	GtkWidget *dialog, *target_name_entry, *ok_button;
 	GtkWidget *target_type_combo, *groups_view;
+	GtkTreePath *root;
 	GtkListStore *types_store;
 	GtkCellRenderer *renderer;
 	gint response;
@@ -1304,15 +1292,18 @@ anjuta_pm_project_new_target (ProjectManagerPlugin *plugin,
 	else
 		gtk_widget_set_sensitive (ok_button, FALSE);
 
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							groups_view,
+	root = gbf_project_model_get_project_root_group (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (groups_view),
+	                        plugin->view,
+	                       	root,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_TARGET),
 							default_group);
+	gtk_tree_path_free (root);
 	gtk_widget_show (groups_view);
 
 	/* setup target types combo box */
-	types_store = build_types_store (plugin->project);
+	types_store = build_types_store (plugin->project, ANJUTA_PROJECT_TARGET);
 	gtk_combo_box_set_model (GTK_COMBO_BOX (target_type_combo), 
 							GTK_TREE_MODEL (types_store));
 
@@ -1435,6 +1426,7 @@ anjuta_pm_project_new_module (ProjectManagerPlugin *plugin,
 	GtkWidget *ok_button, *new_button;
 	GtkWidget *targets_view;
 	GtkWidget *modules_view;
+	GtkTreePath *root;
 	gint response;
 	gboolean finished = FALSE;
 	GList* new_modules = NULL;
@@ -1452,17 +1444,23 @@ anjuta_pm_project_new_module (ProjectManagerPlugin *plugin,
 	new_button = GTK_WIDGET (gtk_builder_get_object (gui, "new_package_button"));
 	ok_button = GTK_WIDGET (gtk_builder_get_object (gui, "ok_module_button"));
 
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							targets_view,
+	root = gbf_project_model_get_project_root_group (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (targets_view),
+	                        plugin->view,
+	                       	root,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_MODULE),
 							default_target);
+	gtk_tree_path_free (root);
 	gtk_widget_show (targets_view);
-	setup_nodes_treeview (gbf_project_view_get_model (plugin->view),
-							modules_view,
+	root = gbf_project_model_get_project_root (gbf_project_view_get_model (plugin->view));
+	setup_nodes_treeview (GBF_PROJECT_VIEW (modules_view),
+	                        plugin->view,
+	                        root,
 							module_filter_func,
 							NULL,
 							NULL);
+	gtk_tree_path_free (root);
 	gtk_widget_show (modules_view);
 	module_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (modules_view));
 	gtk_tree_selection_set_mode (module_selection, GTK_SELECTION_MULTIPLE);
